@@ -41,7 +41,7 @@ LDAPObject_rename(LDAPObject *self, PyObject *args)
 	rc = ldap_rename(self->ldap, dn, newrdn, newparent, deleteoldrdn, sctrls, cctrls, &msgid);
 	LDAP_END_ALLOW_THREADS
 	if (rc != LDAP_SUCCESS) {
-		PyErr_SetString(LDAPError, ldap_err2string(rc));
+		PyErr_Format(LDAPError, "%s (%d)", ldap_err2string(rc), rc);
 		return NULL;
 	}
 	return PyLong_FromLong(msgid);
